from netbox.api.routers import NetBoxRouter
from . import views

router = NetBoxRouter()

router.register('configuration-item', views.ConfigurationItemViewSet)
router.register('service', views.ItilServiceViewSet)
router.register('application', views.ApplicationViewSet)
router.register('relation', views.RelationViewSet)
router.register('pentest', views.PenTestViewSet)

urlpatterns = router.urls
app_name = 'nb_itsm-api'



import django_tables2 as tables

from netbox.tables import NetBoxTable, ToggleColumn , columns

from . import models


class ItilServiceTable(NetBoxTable):

    name = tables.LinkColumn(verbose_name="Service")
    slug = tables.LinkColumn(verbose_name="Slug")
    id = ToggleColumn()
    type = tables.Column(verbose_name="Type")
    status = tables.Column(verbose_name="Status")

    class Meta(NetBoxTable.Meta):
        model = models.Service
        fields = (
            "name",
            "slug",
            "type",
            "status",
        )

class ConfigurationItemsTable(NetBoxTable):
    id = ToggleColumn()
    assigned_object = tables.LinkColumn(verbose_name="Configuration Item")
    obj_type = tables.Column(verbose_name="Type")
    actions = columns.ActionsColumn(actions=("delete",))

    class Meta(NetBoxTable.Meta):
        model = models.ConfigurationItem
        fields = (
            "id",
            "assigned_object",
            "obj_type"
        )

class VulnTable(NetBoxTable):
    id = ToggleColumn()

    class Meta(NetBoxTable.Meta):
        model = models.PenTest
        fields = (
            "id",
            "ticket",
            "date",
            "status",
        )

class ApplicationTable(NetBoxTable):

    name = tables.LinkColumn(verbose_name="Application")
    id = ToggleColumn()

    class Meta(NetBoxTable.Meta):
        model = models.Application
        fields = [
            "name",
            "version",
         ]

class RelationTable(NetBoxTable):
    id = ToggleColumn()

    service = tables.LinkColumn(verbose_name="Service")
    source = tables.LinkColumn(verbose_name="Source")
    destination = tables.LinkColumn(verbose_name="Destination")

    class Meta(NetBoxTable.Meta):
        model = models.Relation
        fields = [
            "service",
            "source",
            "source_shape",
            "destination",
            "destination_shape",
            "connector_shape",
            "link_text",
        ]


from rest_framework import serializers
from django.contrib.contenttypes.models import ContentType

from netbox.api.fields import ContentTypeField
from utilities.api import get_serializer_for_model
from tenancy.api.serializers import TenantSerializer, TenantGroupSerializer
from dcim.api.serializers import DeviceSerializer
from virtualization.api.serializers import VirtualMachineSerializer
from ipam.api.serializers import ServiceSerializer as IpamServiceSerializer

from netbox.api.serializers import NetBoxModelSerializer

from nb_itsm import models
from nb_itsm import choices


class ApplicationSerializer(NetBoxModelSerializer):
    id = serializers.IntegerField(read_only=True)
    name = serializers.CharField()
    display = serializers.SerializerMethodField("get_display")
    version = serializers.CharField()
    devices = DeviceSerializer(many=True, required=False, allow_null=True, nested=True)
    vm = VirtualMachineSerializer(many=True, required=False, allow_null=True, nested=True)
    ipam_services = IpamServiceSerializer(many=True, required=False, allow_null=True, nested=True)

    def get_display(self, obj):
        return f"{obj}"

    class Meta:
        model = models.Application
        fields = [
            "id",
            "display",
            "name",
            "version",
            "devices",
            "vm",
            "ipam_services",
        ]

    def create(self, validated_data):
        devices = validated_data.pop("devices", None)
        virtual_machines = validated_data.pop("vm", None)
        ipam_services = validated_data.pop("ipam_services", None)

        application = super().create(validated_data)

        if devices is not None:
            application.devices.set(devices)
        if virtual_machines is not None:
            application.vm.set(virtual_machines)
        if ipam_services is not None:
            application.ipam_services.set(ipam_services)

        return application

    def update(self, instance, validated_data):
        devices = validated_data.pop("devices", None)
        virtual_machines = validated_data.pop("vm", None)
        ipam_services = validated_data.pop("ipam_services", None)

        application = super().update(instance, validated_data)

        if devices is not None:
            application.devices.set(devices)
        if virtual_machines is not None:
            application.vm.set(virtual_machines)
        if ipam_services is not None:
            application.ipam_services.set(ipam_services)

        return application


class ConfigurationItemSerializer(serializers.Serializer):

    name = serializers.CharField(read_only=True)
    display = serializers.SerializerMethodField("get_display")
    service = serializers.CharField(source="service.name", required=False)

    id = serializers.IntegerField(read_only=True)
    service_id = serializers.IntegerField(
        source="service.id",
        required=True,
    )

    assigned_object_type = ContentTypeField(
        queryset=ContentType.objects.filter(choices.OBJECT_ASSIGNMENT_MODELS),
        required=True,
        allow_null=True,
    )
    assigned_object = serializers.SerializerMethodField(read_only=True)

    assigned_object_id = serializers.IntegerField(source="assigned_object.id", write_only=True)

    def get_display(self, obj):
        return obj.name

    def get_assigned_object(self, obj):
        if obj.assigned_object is None:
            return None
        serializer = get_serializer_for_model(obj.assigned_object)
        context = {"request": self.context["request"]}
        return serializer(obj.assigned_object, nested=True, context=context).data

    class Meta:
        model = models.ConfigurationItem
        fields = [
            "id",
            "display",
            "name",
            "service",
            "assigned_object_type",
            "assigned_object",
            "assigned_object_id",
        ]


class ServiceSerializer(NetBoxModelSerializer):

    name = serializers.CharField()
    slug = serializers.CharField()
    type = serializers.CharField()
    status = serializers.CharField()
    display = serializers.SerializerMethodField("get_display")
    clients = TenantSerializer(many=True, required=False, allow_null=True, nested=True)
    client_groups = TenantGroupSerializer(many=True, required=False, allow_null=True, nested=True)
    comments = serializers.CharField()

    def get_display(self, obj):
        return obj.name

    def create(self, validated_data):
        clients = validated_data.pop("clients", None)
        client_groups = validated_data.pop("client_groups", None)

        service = super().create(validated_data)

        if clients is not None:
            service.clients.set(clients)
        if client_groups is not None:
            service.client_groups.set(client_groups)

        return service

    def update(self, instance, validated_data):
        clients = validated_data.pop("clients", None)
        client_groups = validated_data.pop("client_groups", None)

        service = super().update(instance, validated_data)

        if clients is not None:
            service.clients.set(clients)
        if client_groups is not None:
            service.client_groups.set(client_groups)

        return service

    class Meta:
        model = models.Service
        fields = [
            "id",
            "display",
            "name",
            "slug",
            "type",
            "status",
            "clients",
            "client_groups",
            "comments"
        ]


class RelationSerializer(serializers.Serializer):
    id = serializers.IntegerField(read_only=True)
    service = serializers.SlugRelatedField(
        slug_field="name", queryset=models.Service.objects.all()
    )

    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.Relation
        fields = [
            "id",
            "display",
            "name",
            "service",
            "source",
            "source_shape",
            "destination",
            "destination_shape",
            "connector_shape",
            "link_text",
        ]

class PenTestSerializer(serializers.Serializer):
    id = serializers.IntegerField(read_only=True)
    service = serializers.SlugRelatedField(
        slug_field="name", queryset=models.Service.objects.all()
    )


    class Meta:
        model = models.PenTest
        fields = [
            "id",
            "service",
            "status",
            "comments",
            "date",
            "ticket",
            "report_link"
        ]



from django.urls import path, include
from utilities.urls import get_model_urls

from django.conf import settings
from packaging import version


NETBOX_CURRENT_VERSION = version.parse(settings.VERSION)


from . import views
from . import models

app_name = 'nb_itsm'

urlpatterns = [
    path('service/', views.ServiceListView.as_view(), name='service_list'),
    path('service/<int:pk>/', include(get_model_urls(app_name, 'service'))),
    path('service/<int:pk>/', views.ServiceView.as_view(), name='service'),
    path('service/add/', views.ServiceEditView.as_view(), name='service_add'),
    path('service/<int:pk>/edit/', views.ServiceEditView.as_view(), name='service_edit'),
    path('service/<int:pk>/delete/', views.ServiceDeleteView.as_view(), name='service_delete'),
    path('service/import/', views.ServiceImportView.as_view(), name='service_import'),
    path('service/edit/', views.ServiceBulkEditView.as_view(), name='service_bulk_edit'),
    path('service/delete/', views.ServiceBulkDeleteView.as_view(), name='service_bulk_delete'),

    path('relation/', views.RelationListView.as_view(), name='relation_list'),
    path('relation/<int:pk>/', include(get_model_urls(app_name, 'relation'))),
    path('relation/<int:pk>/', views.RelationView.as_view(), name='relation'),
    path('relation/add', views.RelationEditView.as_view(), name='relation_add'),
    path('relation/<int:pk>/edit/', views.RelationEditView.as_view(), name='relation_edit'),
    path('relation/<int:pk>/delete/', views.RelationDeleteView.as_view(), name='relation_delete'),

    path('configuration-item/add', views.ICCreateView.as_view(), name='configurationitem_add'),
    path('configuration-item/<int:pk>/', include(get_model_urls(app_name, 'configurationitem'))),
    path('configuration-item/<int:pk>/edit/', views.ICCreateView.as_view(), name='configurationitem_edit'),
    path('configuration-item/<int:pk>/delete/', views.ICDeleteView.as_view(), name='configurationitem_delete'),

    path('pentest/add', views.PenTestEditView.as_view(), name='pentest_add'),
    path('pentest/<int:pk>/edit/', views.PenTestEditView.as_view(), name='pentest_edit'),
    path('pentest/<int:pk>/delete/', views.PenTestDeleteView.as_view(), name='pentest_delete'),
    path('pentest/<int:pk>/', include(get_model_urls(app_name, 'pentest'))),

    path('application/', views.ApplicationListView.as_view(), name='application_list'),
    path('application/<int:pk>/', include(get_model_urls(app_name, 'application'))),
    path("application/<int:pk>/", views.ApplicationView.as_view(), name="application"),
    path('application/add', views.ApplicationEditView.as_view(), name='application_add'),
    path('application/<int:pk>/edit/', views.ApplicationEditView.as_view(), name='application_edit'),
    path('application/<int:pk>/delete/', views.ApplicationDeleteView.as_view(), name='application_delete'),
    path('application/import/', views.ApplicationImportView.as_view(), name='application_import'),
    path('application/edit/', views.ApplicationBulkEditView.as_view(), name='application_bulk_edit'),
    path('application/delete/', views.ApplicationBulkDeleteView.as_view(), name='application_bulk_delete'),
]




from django.test import TestCase

from utilities.testing import ChangeLoggedFilterSetTests
from virtualization.models import VirtualMachine, ClusterType, Cluster
from dcim.models import Device, DeviceRole, Manufacturer, DeviceType, Site
from ipam.choices import ServiceProtocolChoices

from nb_itsm.models import Application
from nb_itsm.filtersets import ApplicationFilterSet


class ApplicationFilterTestCase(TestCase, ChangeLoggedFilterSetTests):
    queryset = Application.objects.all()
    filterset = ApplicationFilterSet


    @classmethod
    def setUpTestData(cls):
        cls.ignore_fields = [ "vm", "devices" ]
        site = Site.objects.create(name="Site 1")
        manufacturer = Manufacturer.objects.create(name='Manufacturer 1')
        device_type = DeviceType.objects.create(manufacturer=manufacturer)
        device_role = DeviceRole.objects.create(name='Device Role 1')

        cls.devices = (
            Device(name='Device 1', device_type=device_type, role=device_role, site=site),
            Device(name='Device 2', device_type=device_type, role=device_role, site=site),
            Device(name='Device 3', device_type=device_type, role=device_role, site=site),
        )
        Device.objects.bulk_create(cls.devices)

        clustertype = ClusterType.objects.create(name='Cluster Type 1')
        cluster = Cluster.objects.create(name='Cluster 1', type=clustertype)

        cls.virtual_machines = (
            VirtualMachine(name="Virtual Machine 1", cluster=cluster),
            VirtualMachine(name="Virtual Machine 2", cluster=cluster),
            VirtualMachine(name="Virtual Machine 3", cluster=cluster),
        )
        VirtualMachine.objects.bulk_create(cls.virtual_machines)

        applications = (
            Application(name="SSH", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[22], version="2.0"),
            Application(name="Telnet", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[23], version="42"),
            Application(name="HTTPS", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[443, 8443], version="23"),
            Application(name="SNMP", protocol=ServiceProtocolChoices.PROTOCOL_UDP, ports=[161], version="3"),
        )
        Application.objects.bulk_create(applications)

        applications[0].devices.set([cls.devices[0], cls.devices[1]])
        applications[1].devices.set([cls.devices[1], cls.devices[2]])

        applications[0].vm.set([cls.virtual_machines[0], cls.virtual_machines[1]])
        applications[1].vm.set([cls.virtual_machines[1], cls.virtual_machines[2]])

    def test_name(self):
        params = {'name': ['SSH', 'HTTPS']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_protocol(self):
        params = {'protocol': ServiceProtocolChoices.PROTOCOL_TCP}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)

    def test_devices(self):
        params = {'devices': [self.devices[0].name, self.devices[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
        params = {'devices': [self.devices[0].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
        params = {'devices_id': [self.devices[0].pk, self.devices[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
        params = {'devices_id': [self.devices[0].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)

    def test_virtual_machines(self):
        params = {'virtual_machines': [self.virtual_machines[0].name, self.virtual_machines[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
        params = {'virtual_machines': [self.virtual_machines[0].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
        params = {'virtual_machines_id': [self.virtual_machines[0].pk, self.virtual_machines[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
        params = {'virtual_machines_id': [self.virtual_machines[0].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)



# Generated by Django 5.0.9 on 2024-11-08 14:20

import django.core.validators
import django.db.models.deletion
import re
import taggit.managers
import utilities.json
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("dcim", "0191_module_bay_rebuild"),
        ("extras", "0121_customfield_related_object_filter"),
        ("ipam", "0070_vlangroup_vlan_id_ranges"),
        ("tenancy", "0015_contactassignment_rename_content_type"),
        ("virtualization", "0040_convert_disk_size"),
    ]

    operations = [
        migrations.CreateModel(
            name="Application",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("version", models.CharField(max_length=100)),
                (
                    "devices",
                    models.ManyToManyField(
                        blank=True, related_name="nb_itsm_uses_apps", to="dcim.device"
                    ),
                ),
                (
                    "ipam_services",
                    models.ManyToManyField(
                        blank=True, related_name="nb_itsm_uses_apps", to="ipam.service"
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
                (
                    "vm",
                    models.ManyToManyField(
                        blank=True,
                        related_name="nb_itsm_uses_apps",
                        to="virtualization.virtualmachine",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ConfigurationItem",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    "assigned_object_id",
                    models.PositiveIntegerField(blank=True, null=True),
                ),
                (
                    "assigned_object_type",
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to=models.Q(
                            models.Q(
                                models.Q(("app_label", "dcim"), ("model", "device")),
                                models.Q(
                                    ("app_label", "circuits"), ("model", "circuit")
                                ),
                                models.Q(
                                    ("app_label", "virtualization"),
                                    ("model", "virtualmachine"),
                                ),
                                models.Q(
                                    ("app_label", "nb_itsm"), ("model", "application")
                                ),
                                _connector="OR",
                            )
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="nb_itsm_object_models",
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Service",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                (
                    "slug",
                    models.CharField(
                        default=uuid.uuid4,
                        max_length=50,
                        unique=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["IGNORECASE"],
                                message="Only alphanumeric characters and underscores are allowed.",
                                regex="^[a-z0-9_-]+$",
                            ),
                            django.core.validators.RegexValidator(
                                flags=re.RegexFlag["IGNORECASE"],
                                inverse_match=True,
                                message="Double underscores are not permitted in custom field names.",
                                regex="__",
                            ),
                        ],
                    ),
                ),
                ("status", models.CharField(max_length=100)),
                ("type", models.CharField(max_length=100)),
                ("comments", models.TextField(blank=True)),
                (
                    "client_groups",
                    models.ManyToManyField(
                        related_name="nb_itsm_services", to="tenancy.tenantgroup"
                    ),
                ),
                (
                    "clients",
                    models.ManyToManyField(
                        related_name="nb_itsm_services", to="tenancy.tenant"
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Relation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ("source_shape", models.IntegerField()),
                ("destination_shape", models.IntegerField()),
                ("connector_shape", models.IntegerField()),
                ("link_text", models.CharField(max_length=100)),
                (
                    "destination",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="nb_itsm_destination",
                        to="nb_itsm.configurationitem",
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="nb_itsm_source",
                        to="nb_itsm.configurationitem",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
                (
                    "service",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="nb_itsm_relationships",
                        to="nb_itsm.service",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="PenTest",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ("comments", models.TextField(blank=True)),
                ("status", models.IntegerField()),
                ("date", models.DateField()),
                ("ticket", models.CharField(max_length=100)),
                ("report_link", models.CharField(max_length=100)),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
                (
                    "service",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="nb_itsm_pentest_reports",
                        to="nb_itsm.service",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="configurationitem",
            name="service",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="nb_itsm_configuration_items",
                to="nb_itsm.service",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="configurationitem",
            unique_together={("service", "assigned_object_type", "assigned_object_id")},
        ),
    ]



from utilities.testing import APIViewTestCases, create_tags

from virtualization.models import VirtualMachine, ClusterType, Cluster
from dcim.models import Device, DeviceRole, Manufacturer, DeviceType, Site
from ipam.choices import ServiceProtocolChoices

from nb_itsm.tests.custom import APITestCase
from nb_itsm.models import Application


class ApplicationAPITestCase(
    APITestCase,
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
):
    model = Application

    @classmethod
    def setUpTestData(cls):
        tags = create_tags("Alpha", "Bravo", "Charlie")

        site = Site.objects.create(name="Site 1")
        manufacturer = Manufacturer.objects.create(name='Manufacturer 1')
        device_type = DeviceType.objects.create(manufacturer=manufacturer)
        device_role = DeviceRole.objects.create(name='Device Role 1')

        devices = (
            Device(name='Device 1', device_type=device_type, role=device_role, site=site),
            Device(name='Device 2', device_type=device_type, role=device_role, site=site),
            Device(name='Device 3', device_type=device_type, role=device_role, site=site),
        )
        Device.objects.bulk_create(devices)

        clustertype = ClusterType.objects.create(name='Cluster Type 1')
        cluster = Cluster.objects.create(name='Cluster 1', type=clustertype)

        virtual_machines = (
            VirtualMachine(name="Virtual Machine 1", cluster=cluster),
            VirtualMachine(name="Virtual Machine 2", cluster=cluster),
            VirtualMachine(name="Virtual Machine 3", cluster=cluster),
        )
        VirtualMachine.objects.bulk_create(virtual_machines)

        applications = (
            Application(name="SSH", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[22], version="2.0"),
            Application(name="Telnet", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[23], version="42"),
            Application(name="HTTPS", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[443, 8443], version="23"),
        )
        Application.objects.bulk_create(applications)

        cls.create_data = [
            {"name": "SMTP", "protocol": ServiceProtocolChoices.PROTOCOL_TCP, "ports": [25, 587], "version": "1.0", "devices": [devices[2].pk]},
            {"name": "SNMP", "protocol": ServiceProtocolChoices.PROTOCOL_UDP, "ports": [161], "version": "3", "devices": [devices[1].pk, devices[2].pk]},
            {"name": "SNMP Trap", "protocol": ServiceProtocolChoices.PROTOCOL_UDP, "ports": [162], "version": "3", "vm": [virtual_machines[0].pk, virtual_machines[1].pk]},
        ]

        cls.bulk_update_data = {
            "protocol": ServiceProtocolChoices.PROTOCOL_SCTP,
            "vm": [virtual_machines[0].pk, virtual_machines[1].pk, virtual_machines[2].pk],
            "devices": [devices[0].pk, devices[1].pk],
        }

        cls.brief_fields = ['devices', 'display', 'id', 'name', 'ports', 'protocol', 'version', 'vm']



from netbox.plugins import PluginMenu, PluginMenuButton, PluginMenuItem


from django.conf import settings

plugin_settings = settings.PLUGINS_CONFIG["nb_itsm"]

menu_buttons = (
    PluginMenuItem(
        permissions=["nb_itsm.view_service"],
        link="plugins:nb_itsm:service_list",
        link_text="Service Catalog",
    ),
    PluginMenuItem(
        permissions=["nb_itsm.view_application"],
        link="plugins:nb_itsm:application_list",
        link_text="Applications",
    ),
)

if plugin_settings.get("top_level_menu"):
    menu = PluginMenu(
        label="IT Service management",
        groups=(("ITSM", menu_buttons),),
        icon_class="mdi mdi-cog-outline",
    )
else:
    menu_items = menu_buttons


from django.db.models.signals import post_save,post_delete
from django.dispatch import receiver
from django.contrib.contenttypes.models import ContentType

from dcim.models import Device
from virtualization.models import VirtualMachine

from . import models

def handle_configuration_item_assigned_object_delete(sender, instance, *args, **kwargs):
    related_ics = models.ConfigurationItem.objects.filter(
        assigned_object_type=ContentType.objects.get_for_model(instance),
         assigned_object_id=instance.id)
    for configuration_items in related_ics:
        configuration_items.delete()


device_delete = receiver(post_delete, sender=Device)(handle_configuration_item_assigned_object_delete)
vm_delete = receiver(post_delete, sender=VirtualMachine)(handle_configuration_item_assigned_object_delete)
app_delete = receiver(post_delete, sender=models.Application)(handle_configuration_item_assigned_object_delete)



from netbox.api.viewsets import NetBoxModelViewSet

from nb_itsm import models
from nb_itsm import filtersets
from . import serializers

class ConfigurationItemViewSet(NetBoxModelViewSet):
    queryset = models.ConfigurationItem.objects.all()
    serializer_class = serializers.ConfigurationItemSerializer
    filterset_class = filtersets.ConfigurationItemFilter

class ItilServiceViewSet(NetBoxModelViewSet):
    queryset = models.Service.objects.all()
    serializer_class = serializers.ServiceSerializer

class ApplicationViewSet(NetBoxModelViewSet):
    queryset = models.Application.objects.all()
    serializer_class = serializers.ApplicationSerializer

class RelationViewSet(NetBoxModelViewSet):
    queryset = models.Relation.objects.all()
    serializer_class = serializers.RelationSerializer

class PenTestViewSet(NetBoxModelViewSet):
    queryset = models.PenTest.objects.all()
    serializer_class = serializers.PenTestSerializer



from utilities.testing import ViewTestCases
from utilities.testing import create_tags

from nb_itsm.tests.custom import ModelViewTestCase
from nb_itsm.models import Service


class ServiceViewTestCase(
    ModelViewTestCase,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.BulkImportObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    model = Service

    @classmethod
    def setUpTestData(cls):
        services = (
            Service(name="Test Service 1", comments="Test Comment 1"),
            Service(name="Test Service 2", comments="Test Comment 2"),
            Service(name="Test Service 3", comments="Test Comment 3"),
        )
        Service.objects.bulk_create(services)

        cls.form_data = {
            "name": "Test Service 4",
        }

        cls.csv_data = (
            "name",
            "Service 4",
            "Service 5",
            "Service 6",
        )

        cls.csv_update_data = (
            "id,name",
            f"{services[0].pk},New Test Service 1",
            f"{services[1].pk},New Test Service 2",
        )

    maxDiff = None



from utilities.testing import APIViewTestCases, create_tags
from tenancy.models import Tenant

from nb_itsm.tests.custom import APITestCase
from nb_itsm.models import Service


class ServiceAPITestCase(
    APITestCase,
    APIViewTestCases.GetObjectViewTestCase,
    APIViewTestCases.ListObjectsViewTestCase,
    APIViewTestCases.CreateObjectViewTestCase,
    APIViewTestCases.UpdateObjectViewTestCase,
    APIViewTestCases.DeleteObjectViewTestCase,
):
    model = Service

    @classmethod
    def setUpTestData(cls):
        tags = create_tags("Alpha", "Bravo", "Charlie")

        services = (
            Service(name="Test Service 1", comments="Test Comment 1",),
            Service(name="Test Service 2", comments="Test Comment 2",),
            Service(name="Test Service 3", comments="Test Comment 3",),
        )
        Service.objects.bulk_create(services)

        tenants = (
            Tenant(name="Tenant 1", slug="tenant-1"),
            Tenant(name="Tenant 2", slug="tenant-2"),
            Tenant(name="Tenant 3", slug="tenant-3"),
        )
        Tenant.objects.bulk_create(tenants)

        cls.create_data = [
            {"name": "Test Service 4", "comments": "Test Comment 4", "clients": [tenants[0].pk, tenants[1].pk] },
            {"name": "Test Service 5", "comments": "Test Comment 5", "clients": [tenants[1].pk, tenants[2].pk] },
            {"name": "Test Service 6", "comments": "Test Comment 6", "clients": [tenants[2].pk] },
        ]

        cls.bulk_update_data = {
            "clients": [tenants[1].pk]
        }

        cls.brief_fields = ['clients', 'comments', 'display', 'id', 'name']



from . import models
from . import filtersets
from . import forms
from . import tables

from netbox.views import generic
from tenancy.tables import TenantTable, TenantGroupTable
from virtualization.models import VirtualMachine
from dcim.models import Device
from dcim.tables import DeviceTable
from virtualization.tables import VirtualMachineTable
from ipam.tables import ServiceTable as IpamServiceTable
from utilities.views import ViewTab, register_model_view
from netbox.views.generic import ObjectChildrenView
from ipam.models import Service as IpamService

#
# Service Views ###############################################################
#

class ServiceListView(generic.ObjectListView):
    queryset = models.Service.objects.all()
    table = tables.ItilServiceTable
    filterset = filtersets.ItilServiceFilterSet
    filterset_form = forms.ServiceFilterForm

class ServiceView(generic.ObjectView):
    queryset = models.Service.objects.all()

    def get_extra_context(self, request, instance):
        tenants_table = TenantTable(instance.clients.all())
        tenant_groups_table = TenantGroupTable(instance.client_groups.all())

        vuln_table = tables.VulnTable(instance.nb_itsm_pentest_reports.all())

        data = {
                "tenant_table" : tenants_table,
                "tenant_groups_table" : tenant_groups_table,
                "vuln_table" : vuln_table,
            }
        return data

@register_model_view(models.Service, 'contacts')
class ItilServiceContactsView(ObjectChildrenView):
    queryset = models.Service.objects.all()

@register_model_view(models.Service, name='configuration_items')
class ServiceConfigurationItemView(generic.ObjectChildrenView):
    queryset = models.Service.objects.all()
    table = tables.ConfigurationItemsTable
    template_name = "nb_itsm/service-configuration-item-view.html"
    tab = ViewTab(label='Configuration Items', badge=lambda obj: obj.nb_itsm_configuration_items.all().count(), hide_if_empty=True)

    def get_children(self, request, parent):
            children = parent.nb_itsm_configuration_items.all()
            return children


@register_model_view(models.Service, name='relationships')
class ServiceRelationView(generic.ObjectChildrenView):
    queryset = models.Service.objects.all()
    child_model = models.Relation
    table = tables.RelationTable
    template_name = "nb_itsm/service_Relation_view.html"
    tab = ViewTab(
        label='CI Relationships',
        badge=lambda obj: obj.nb_itsm_relationships.all().count(),
        hide_if_empty=True
    )

    def get_children(self, request, parent):
            children = parent.nb_itsm_relationships.all()
            return children

    def get_extra_context(self, request, instance):
        relations = tables.RelationTable(instance.nb_itsm_relationships.all())
        data = {
                "table" : relations,
            }
        return data

@register_model_view(models.Service, name='diagram')
class ServiceDiagramView(generic.ObjectChildrenView):
    queryset = models.Service.objects.all()
    child_model = models.Service
    table = tables.ItilServiceTable
    template_name = "nb_itsm/service_diagram_view.html"
    tab = ViewTab(label='Diagram')

    def get_children(self, request, parent):
            children = parent.nb_itsm_relationships.all()
            return children

class ServiceEditView(generic.ObjectEditView):
    queryset = models.Service.objects.all()
    form  = forms.ItilServiceForm

class ServiceImportView(generic.BulkImportView):
    queryset = models.Service.objects.all()
    model_form = forms.ItilServiceImportForm
    table = tables.ItilServiceTable

class ServiceBulkEditView(generic.BulkEditView):
    queryset = models.Service.objects.all()
    filterset = filtersets.ItilServiceFilterSet
    table = tables.ItilServiceTable
    form = forms.ServiceBulkEditForm

class ServiceBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Service.objects.all()
    table = tables.ItilServiceTable

class ServiceDeleteView(generic.ObjectDeleteView):
    queryset = models.Service.objects.all()


class ICCreateView(generic.ObjectEditView):
    queryset = models.ConfigurationItem.objects.all()
    form = forms.ConfigurationItemForm

    def alter_object(self, obj, request, url_args, url_kwargs):
        if 'device' in request.POST:
            try:
                obj.assigned_object =  Device.objects.get(pk=request.POST['device'])
            except (ValueError, Device.DoesNotExist):
                pass

        if 'virtual_machine' in request.POST:
            try:
                obj.assigned_object =  VirtualMachine.objects.get(pk=request.POST['virtual_machine'])
            except (ValueError, Device.DoesNotExist):
                pass
        if 'application' in request.POST:
            try:
                obj.assigned_object =  models.Application.objects.get(pk=request.POST['application'])
            except (ValueError, Device.DoesNotExist):
                pass

        return obj

#
# Pentest Extra Views ###########################################################
#

class PenTestEditView(generic.ObjectEditView):
    queryset = models.PenTest.objects.all()
    form = forms.PenTestForm

class PenTestDeleteView(generic.ObjectDeleteView):
    queryset = models.PenTest.objects.all()

class RelationDeleteView(generic.ObjectDeleteView):
    queryset = models.Relation.objects.all()

class ICDeleteView(generic.ObjectDeleteView):
    queryset = models.ConfigurationItem.objects.all()

#
# Application Views ###########################################################
#

class ApplicationListView(generic.ObjectListView):
    queryset = models.Application.objects.all()
    table = tables.ApplicationTable
    filterset = filtersets.ApplicationFilterSet
    filterset_form = forms.ApplicationFilterForm

class ApplicationView(generic.ObjectView):
    queryset = models.Application.objects.all()

@register_model_view(models.Application, name='devices')
class ApplicationDevicesView(generic.ObjectChildrenView):
    queryset = models.Application.objects.all()
    child_model = Device
    table = DeviceTable
    template_name = "nb_itsm/application_objs.html"
    tab = ViewTab(label='Devices', badge=lambda obj: obj.devices.all().count(), hide_if_empty=True)

    def get_children(self, request, parent):
            children = parent.devices.all()
            return children

    def get_extra_context(self, request, instance):
        device_table = DeviceTable(instance.devices.all())
        device_table.exclude = ('actions',)
        data = {
                "table" : device_table,
            }
        return data

@register_model_view(models.Application, name='vms')
class ApplicationVMsView(generic.ObjectChildrenView):
    queryset = models.Application.objects.all()
    child_model = VirtualMachine
    table = VirtualMachineTable
    template_name = "nb_itsm/application_objs.html"
    tab = ViewTab(label='Virtual Machines', badge=lambda obj: obj.vm.all().count(), hide_if_empty=True)

    def get_children(self, request, parent):
            children = parent.vm.all()
            return children

    def get_extra_context(self, request, instance):
        vm_table = VirtualMachineTable(instance.vm.all())
        vm_table.exclude = ('actions',)
        data = {
                "table" : vm_table,
            }
        return data

@register_model_view(models.Application, name='ipam_services')
class ApplicationIpamServicesView(generic.ObjectChildrenView):

    queryset = models.Application.objects.all()
    child_model = IpamService
    table = IpamServiceTable
    template_name = "nb_itsm/application_objs.html"
    tab = ViewTab(label='IPAM Services', badge=lambda obj: obj.ipam_services.all().count(), hide_if_empty=True)

    def get_children(self, request, parent):
            children = parent.ipam_services.all()
            return children
    def get_extra_context(self, request, instance):
        ipam_service_table = IpamServiceTable(instance.ipam_services.all())
        ipam_service_table.exclude = ('actions',)
        data = {
                "table" : ipam_service_table,
            }
        return data

class ApplicationEditView(generic.ObjectEditView):
    queryset = models.Application.objects.all()
    form = forms.ApplicationForm

class ApplicationImportView(generic.BulkImportView):
    queryset = models.Application.objects.all()
    model_form = forms.ApplicationImportForm
    table = tables.ApplicationTable

class ApplicationBulkEditView(generic.BulkEditView):
    queryset = models.Application.objects.all()
    filterset = filtersets.ApplicationFilterSet
    table = tables.ApplicationTable
    form = forms.ApplicationBulkEditForm

class ApplicationBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Application.objects.all()
    table = tables.ApplicationTable

class ApplicationDeleteView(generic.ObjectDeleteView):
    queryset = models.Application.objects.all()

#
# Relation Views ##############################################################
#

class RelationListView(generic.ObjectListView):
    queryset = models.Relation.objects.all()
    table = tables.RelationTable
    filterset = filtersets.RelationFilter
    filterset_form = forms.RelationFilterForm

class RelationView(generic.ObjectView):
    queryset = models.Relation.objects.all()

class RelationEditView(generic.ObjectEditView):
    queryset = models.Relation.objects.all()
    form = forms.RelationForm



from utilities.testing import ViewTestCases, create_tags
from virtualization.models import VirtualMachine, ClusterType, Cluster
from dcim.models import Device, DeviceRole, Manufacturer, DeviceType, Site
from ipam.choices import ServiceProtocolChoices

from nb_itsm.tests.custom import ModelViewTestCase
from nb_itsm.models import Application


class ApplicationViewTestCase(
    ModelViewTestCase,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.BulkImportObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    model = Application

    @classmethod
    def setUpTestData(cls):
        site = Site.objects.create(name="Site 1")
        manufacturer = Manufacturer.objects.create(name='Manufacturer 1')
        device_type = DeviceType.objects.create(manufacturer=manufacturer)
        device_role = DeviceRole.objects.create(name='Device Role 1')

        devices = (
            Device(name='Device 1', device_type=device_type, role=device_role, site=site),
            Device(name='Device 2', device_type=device_type, role=device_role, site=site),
            Device(name='Device 3', device_type=device_type, role=device_role, site=site),
        )
        Device.objects.bulk_create(devices)

        clustertype = ClusterType.objects.create(name='Cluster Type 1')
        cluster = Cluster.objects.create(name='Cluster 1', type=clustertype)

        virtual_machines = (
            VirtualMachine(name="Virtual Machine 1", cluster=cluster),
            VirtualMachine(name="Virtual Machine 2", cluster=cluster),
            VirtualMachine(name="Virtual Machine 3", cluster=cluster),
        )
        VirtualMachine.objects.bulk_create(virtual_machines)

        applications = (
            Application(name="SSH", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[22], version="2.0"),
            Application(name="Telnet", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[23], version="42"),
            Application(name="HTTPS", protocol=ServiceProtocolChoices.PROTOCOL_TCP, ports=[443, 8443], version="23"),
            Application(name="SNMP", protocol=ServiceProtocolChoices.PROTOCOL_UDP, ports=[161], version="3"),
        )
        Application.objects.bulk_create(applications)

        cls.form_data = {
            "name": "Test Application 4", "protocol": ServiceProtocolChoices.PROTOCOL_TCP, "ports": "42", "version": "1.0"
        }

        cls.csv_data = (
            "name,protocol,ports,version,devices,vm",
            f"Application 4,tcp,43,1.0,\"{devices[0].name}\",",
            f"Application 5,tcp,44,2.0,{devices[2].name},{virtual_machines[1].name}",
            f"Application 6,tcp,45,2.0,,{virtual_machines[1].name}",
        )

        cls.csv_update_data = (
            "id,name",
            f"{applications[0].pk},New Test Application 1",
            f"{applications[1].pk},New Test Application 2",
        )

    maxDiff = None



from django import forms

from dcim.models import Device
from tenancy.models import Tenant, TenantGroup
from virtualization.models import VirtualMachine
from ipam.models import Service as IpamService
from utilities.forms.fields import (
    DynamicModelMultipleChoiceField,
    DynamicModelChoiceField,
    CSVModelMultipleChoiceField,
)
from utilities.forms.widgets import DatePicker

from netbox.forms import (
    NetBoxModelForm,
    NetBoxModelFilterSetForm,
    NetBoxModelBulkEditForm,
    NetBoxModelImportForm,
)

from . import models

class ItilServiceForm(NetBoxModelForm):

    clients = DynamicModelMultipleChoiceField(label="Clients",
        queryset=Tenant.objects.all(),
        required=False,
    )

    client_groups = DynamicModelMultipleChoiceField(label="Client Groups",
        queryset=TenantGroup.objects.all(),
        required=False,
    )

    class Meta:
        model = models.Service
        fields = [
            "name",
            "slug",
            "status",
            "type",
            "clients",
            "client_groups",
            "comments",
        ]

class ApplicationForm(NetBoxModelForm):

    devices = DynamicModelMultipleChoiceField(label="Devices",
        queryset=Device.objects.all(),
        required=False,
    )
    vm = DynamicModelMultipleChoiceField(label="Virtual Machines",
        queryset=VirtualMachine.objects.all(),
        required=False,
    )
    ipam_services = DynamicModelMultipleChoiceField(label="IPAM Services",
        queryset=IpamService.objects.all(),
        required=False,
    )

    class Meta:
        model = models.Application
        fields = [
            "name",
            "version",
            "devices",
            "vm",
            "ipam_services",
        ]

class ConfigurationItemForm(NetBoxModelForm):

    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
    )
    virtual_machine = DynamicModelChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False,
    )

    application = DynamicModelChoiceField(
        queryset=models.Application.objects.all(),
        required=False,
    )

    class Meta:
        model = models.ConfigurationItem
        fields = [
            'service',
        ]


class PenTestForm(NetBoxModelForm):

    class Meta:
        model = models.PenTest
        fields = [
            'service',
            'comments',
            'status',
            'date',
            'ticket',
            "report_link",
        ]

        widgets = {
            'date': DatePicker(),
        }

class RelationForm(NetBoxModelForm):

    source = DynamicModelChoiceField(
        queryset=models.ConfigurationItem.objects.all(),
        required=True,
        label='Source',
        query_params={
            'service_id': '$service',
        },
    )

    destination = DynamicModelChoiceField(
        queryset=models.ConfigurationItem.objects.all(),
        required=True,
        label='Destination',
        query_params={
            'service_id': '$service',
        },
    )

    link_text = forms.CharField(required=False)

    class Meta:
        model = models.Relation
        fields = [
            'service',
            'source',
            'source_shape',
            'destination',
            'destination_shape',
            "connector_shape",
            "link_text",
        ]

class RelationFilterForm(NetBoxModelFilterSetForm):
    model = models.Relation

    class Meta:
        fields = [
            'service',
            'source',
            'source_shape',
            'destination',
            'destination_shape',
            "connector_shape",
            "link_text",
        ]


class ServiceFilterForm(forms.ModelForm):

    q = forms.CharField(
        required=False,
        label='Search'
    )
    clients = DynamicModelMultipleChoiceField(
        label="Clients",
        queryset=Tenant.objects.all(),
        required=False,
    )

    class Meta:
        model = models.Service
        fields = [
            'q',
            'clients',
        ]

class ServiceBulkEditForm(NetBoxModelBulkEditForm):
    model = models.Service

    clients = DynamicModelMultipleChoiceField(
        label="Clients",
        queryset=Tenant.objects.all(),
        required=False,
    )
    comments = forms.Textarea(
        attrs={'class': 'font-monospace'}
    )

    class Meta:
        nullable_fields = (
            "clients",
            "comments",
        )

class ItilServiceImportForm(NetBoxModelImportForm):
    clients = CSVModelMultipleChoiceField(
        label="Clients",
        queryset=Tenant.objects.all(),
        required=False,
        to_field_name="name",
        error_messages={
            "invalid_choice": "Client name not found",
        }
    )

    class Meta:
        model = models.Service
        fields = ["name",
                  "clients",
                  "comments",
          ]


class ApplicationFilterForm(forms.ModelForm):

    q = forms.CharField(
        required=False,
        label='Search'
    )
    devices = DynamicModelMultipleChoiceField(label="Devices",
        queryset=Device.objects.all(),
        required=False,
    )

    vm = DynamicModelMultipleChoiceField(label="Virtual Machines",
        queryset=VirtualMachine.objects.all(),
        required=False,
    )

    ipam_services = DynamicModelMultipleChoiceField(label="IPAM Services",
        queryset=IpamService.objects.all(),
        required=False,
    )

    class Meta:
        model = models.Service
        fields = [
            'q',
            'devices',
            'vm',
            'ipam_services',
        ]

class ApplicationImportForm(NetBoxModelImportForm):

    devices = CSVModelMultipleChoiceField(
        label="Devices",
        queryset=Device.objects.all(),
        required=False,
        to_field_name="name",
        error_messages={
            "invalid_choice": "Device name not found",
        }
    )
    vm = CSVModelMultipleChoiceField(
        label="Virtual Machines",
        queryset=VirtualMachine.objects.all(),
        required=False,
        to_field_name="name",
        error_messages={
            "invalid_choice": "Virtual machine name not found",
        }
    )

    class Meta:
        model = models.Application
        fields = ["name", "version", "devices", "vm"]

class ApplicationBulkEditForm(NetBoxModelBulkEditForm):
    model = models.Application

    devices = DynamicModelMultipleChoiceField(
        label="Devices",
        queryset=Device.objects.all(),
        required=False,
    )
    vm = DynamicModelMultipleChoiceField(
        label="Virtual Machines",
        queryset=VirtualMachine.objects.all(),
        required=False,
    )

    class Meta:
        fields = ["version", "devices", "vm"]
        nullable_fields = ("devices", "vm")



from django.test import TestCase

from tenancy.models import Tenant
from utilities.testing import ChangeLoggedFilterSetTests

from nb_itsm.models import Service
from nb_itsm.filtersets import ItilServiceFilterSet


class ServiceFilterTestCase(TestCase, ChangeLoggedFilterSetTests):
    queryset = Service.objects.all()
    filterset = ItilServiceFilterSet

    @classmethod
    def setUpTestData(cls):
        cls.ignore_fields = ( "clients", )
        cls.tenants = (
            Tenant(name="Tenant 1", slug="tenant-1"),
            Tenant(name="Tenant 2", slug="tenant-2"),
            Tenant(name="Tenant 3", slug="tenant-3"),
        )
        Tenant.objects.bulk_create(cls.tenants)

        cls.services = (
            Service(name="Service 1"),
            Service(name="Service 2"),
            Service(name="Service 3"),
        )
        Service.objects.bulk_create(cls.services)

        cls.services[0].clients.set([cls.tenants[0].pk, cls.tenants[1].pk])
        cls.services[1].clients.set([cls.tenants[1].pk, cls.tenants[2].pk])
        cls.services[2].clients.set([cls.tenants[0].pk])

    def test_name(self):
        params = {'name': ['Service 1', 'Service 2']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_clients(self):
        params = {"clients": [self.tenants[0].name, self.tenants[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
        params = {"clients": [self.tenants[2].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
        params = {"clients_id": [self.tenants[0].pk, self.tenants[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
        params = {"clients_id": [self.tenants[2].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)



import importlib.metadata
from netbox.plugins import PluginConfig
from .version import __version__


class NetboxItsmConfig(PluginConfig):
    name = 'nb_itsm'
    base_url = 'nb-itsm'
    verbose_name = 'ITIL Service Management'
    description = importlib.metadata.metadata('nb_itsm').get('summary', 'pyproject.toml error')
    version = __version__
    author = importlib.metadata.metadata('nb_itsm').get('author', 'pyproject.toml error')
    author_email = 'admin@cispa.de'
    min_version = "4.1.0"
    max_version = "4.4.99"
    required_settings = []
    default_settings = {
        "top_level_menu": True
    }

    def ready(self):
        super().ready()

config = NetboxItsmConfig # noqa



#
import re
import uuid

# Django imports
from django.db import models
from django.urls import reverse
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.utils.translation import gettext_lazy as _

# Netbox imports
from taggit.managers import TaggableManager
from netbox.models import NetBoxModel
from netbox.models.features import ContactsMixin
from virtualization.models import VirtualMachine
from dcim.models import Device
from ipam.models import Service as IpamService
from tenancy.models import Tenant, TenantGroup

# module imports
from . import choices

SERVICE_PORT_MIN = 1
SERVICE_PORT_MAX = 65535
SHAPE_NAMES = [
    "Round Edges",
    "Stadium Shaped",
    "Subroutine Shape",
    "Cylindrical Shape",
    "Circle Shape",
    "asymmetric shape",
    "rhombus",
    "Hexagon",
    "Parallelogram",
    "Trapezoid",
]

class Service(ContactsMixin, NetBoxModel):
    name = models.CharField(
        "Name",
        max_length=100
    )

    slug = models.CharField(
        verbose_name=_('Slug'),
        max_length=50,
        default=uuid.uuid4,
        unique=True,
        help_text=_('Short name for referencing other applications and services'),
        validators=(
            RegexValidator(
                regex=r'^[a-z0-9_-]+$',
                message=_("Only alphanumeric characters and underscores are allowed."),
                flags=re.IGNORECASE
            ),
            RegexValidator(
                regex=r'__',
                message=_("Double underscores are not permitted in custom field names."),
                flags=re.IGNORECASE,
                inverse_match=True
            ),
        )
    )

    status = models.CharField(
        max_length=100,
        choices=choices.ItilServiceStatusChoices
    )
    def get_status_color(self):
        return choices.ItilServiceStatusChoices.colors.get(self.status)

    type = models.CharField(
        max_length=100,
        choices=choices.ItilServiceTypeChoices
    )
    def get_type_color(self):
        return choices.ItilServiceTypeChoices.colors.get(self.type)

    clients = models.ManyToManyField(
        Tenant,
        related_name="nb_itsm_services",
        verbose_name="Clients"
    )

    client_groups = models.ManyToManyField(
        TenantGroup,
        related_name="nb_itsm_services",
        verbose_name="Client Groups"
    )

    comments = models.TextField(blank=True)

    tags = TaggableManager(
        through="extras.TaggedItem",
        related_name="nb_itsm_service_set",
    )

    @property
    def diagram(self):
        mermaid_graph_type: str = "graph TB\n"
        graph = mermaid_graph_type

        open_shape = [
            "(",
            "([",
            "[[",
            "[(",
            "((",
            ">",
            "{",
            "{{",
            "[/",
            "[\\",
        ]

        close_shape = [
            ")",
            "])",
            "]]",
            ")]",
            "))",
            "]",
            "}",
            "}}",
            "/]",
            "\\]",
        ]
        arrow_shape = ["-->", "---", "-.->", "-.-"]
        nodes = {}
        for configuration_item in self.nb_itsm_configuration_items.all():
            node = configuration_item.name.replace(" ", "_")
            graph += f"{node}\n"
            if node not in nodes:
                nodes[node] = configuration_item.get_absolute_url()
        for rel in self.nb_itsm_relationships.all():
            src_node = rel.source.name.replace(" ", "_")
            dest_node = rel.destination.name.replace(" ", "_")
            if src_node not in nodes:
                nodes[src_node] = rel.source.get_absolute_url()
            if dest_node not in nodes:
                nodes[dest_node] = rel.destination.get_absolute_url()

            graph += (
                f"    {src_node}{open_shape[rel.source_shape -1]}"
                + f"{rel.source.name}{close_shape[rel.source_shape -1]}"
                + f" {arrow_shape[rel.connector_shape -1]} "
            )
            if rel.link_text != "":
                graph += f"| {rel.link_text} |"
            graph += (
                f"{dest_node}{open_shape[rel.destination_shape -1]}"
                + f"{rel.destination.name}{close_shape[rel.destination_shape -1]}\n"
            )
        for node in nodes:
            graph += f'click {node} "{nodes[node]}"\n'

        if graph is mermaid_graph_type:
            graph += "empty"

        return graph

    def __str__(self) -> str:
        return f"{self.name}"

    def get_absolute_url(self):
        return reverse("plugins:nb_itsm:service", kwargs={"pk": self.pk})


class Application(NetBoxModel):
    name = models.CharField("Name", max_length=100)
    version = models.CharField("Version", max_length=100)
    devices = models.ManyToManyField(
        Device,
        related_name="nb_itsm_uses_apps",
        verbose_name="Devices",
        blank=True,
    )
    vm = models.ManyToManyField(
        VirtualMachine,
        related_name="nb_itsm_uses_apps",
        verbose_name="Virtual Machines",
        blank=True,
    )
    ipam_services = models.ManyToManyField(
        IpamService,
        related_name="nb_itsm_uses_apps",
        verbose_name="IPAM Services",
        blank=True,
    )
    tags = TaggableManager(
        through="extras.TaggedItem",
        related_name="nb_itsm_application_set",
    )

    def get_absolute_url(self):
        return reverse(
            "plugins:nb_itsm:application",
            kwargs={
                "pk": self.pk,
            }
        )

    def __str__(self) -> str:
        return f"{self.name} - {self.version}"


class ConfigurationItem(NetBoxModel):
    service = models.ForeignKey(
        to=Service, on_delete=models.CASCADE, related_name="nb_itsm_configuration_items"
    )

    assigned_object_type = models.ForeignKey(
        to=ContentType,
        limit_choices_to=choices.OBJECT_ASSIGNMENT_MODELS,
        on_delete=models.CASCADE,
        related_name="nb_itsm_object_models",
        blank=True,
        null=True,
    )
    assigned_object_id = models.PositiveIntegerField(blank=True, null=True)
    assigned_object = GenericForeignKey(
        ct_field="assigned_object_type",
        fk_field="assigned_object_id",
    )

    @property
    def obj_type(self):
        return self.assigned_object_type.name

    @property
    def name(self):
        if self.assigned_object is not None:
            return self.assigned_object.name
        else:
            return ""

    def __str__(self):
        return str(self.assigned_object)

    def get_absolute_url(self):
        url = reverse("plugins:nb_itsm:service", kwargs={"pk": self.service.pk})
        try:
            url = self.assigned_object.get_absolute_url()
        except Exception as e:
            pass
        return url

    def validate_unique(self, exclude=None):
        configuration_items = self.service.nb_itsm_configuration_items.filter(
            assigned_object_type=self.assigned_object_type
        )
        candidate_obj = self.assigned_object
        for obj in configuration_items:
            if candidate_obj == obj.assigned_object:
                raise ValidationError(
                    (f"{self.assigned_object} already in {self.service}")
                )

        super().validate_unique(exclude=exclude)

    class Meta:
        unique_together = [
            ["service", "assigned_object_type", "assigned_object_id"],
        ]


class Relation(NetBoxModel):
    service = models.ForeignKey(
        verbose_name="Service",
        to=Service,
        on_delete=models.CASCADE,
        related_name="nb_itsm_relationships",
    )
    source = models.ForeignKey(
        verbose_name="Source", to=ConfigurationItem, on_delete=models.CASCADE, related_name="nb_itsm_source"
    )
    source_shape = models.IntegerField("Source Shape", choices=choices.ShapeChoices)
    destination = models.ForeignKey(
        verbose_name="Destination", to=ConfigurationItem, on_delete=models.CASCADE, related_name="nb_itsm_destination"
    )
    destination_shape = models.IntegerField(
        "Destination Shape", choices=choices.ShapeChoices
    )
    connector_shape = models.IntegerField(
        "Connector Shape", choices=choices.ConnectorChoices
    )

    link_text = models.CharField("Link Text", max_length=100)

    tags = TaggableManager(
        through="extras.TaggedItem",
        related_name="nb_itsm_relation_set",
    )


    @property
    def source_shape_name(self):
        return SHAPE_NAMES[self.source_shape - 1]

    @property
    def destiny_shape_name(self):
        return SHAPE_NAMES[self.destination_shape - 1]

    @property
    def connector(self):
        arrow_shape = ["-->", "---", "-.->", "-.-"]
        return f"{arrow_shape[self.connector_shape -1]}"

    def get_absolute_url(self):
        return reverse("plugins:nb_itsm:relation", kwargs={"pk": self.pk})

    @property
    def diagram(self):
        graph = "graph TB\n"
        open_shape = [
            "(",
            "([",
            "[[",
            "[(",
            "((",
            ">",
            "{",
            "{{",
            "[/",
            "[\\",
        ]

        close_shape = [
            ")",
            "])",
            "]]",
            ")]",
            "))",
            "]",
            "}",
            "}}",
            "/]",
            "\\]",
        ]
        arrow_shape = ["-->", "---", "-.->", "-.-"]
        nodes = {}
        relations = [ self ]
        for rel in relations:
            src_node = rel.source.name.replace(" ", "_")
            dest_node = rel.destination.name.replace(" ", "_")
            if src_node not in nodes:
                nodes[src_node] = rel.source.get_absolute_url()
            if dest_node not in nodes:
                nodes[dest_node] = rel.destination.get_absolute_url()

            graph += (
                f"    {src_node}{open_shape[rel.source_shape -1]}"
                + f"{rel.source.name}{close_shape[rel.source_shape -1]}"
                + f" {arrow_shape[rel.connector_shape -1]} "
            )
            if rel.link_text != "":
                graph += f"| {rel.link_text} |"
            graph += (
                f"{dest_node}{open_shape[rel.destination_shape -1]}"
                + f"{rel.destination.name}{close_shape[rel.destination_shape -1]}\n"
            )
        for node in nodes:
            graph += f'click {node} "{nodes[node]}"\n'
        return graph

    def __str__(self) -> str:
        arrow_shape = ["-->", "---", "-.->", "-.-"]
        src_node = self.source.name.replace(" ", "_")
        dest_node = self.destination.name.replace(" ", "_")

        return f"{src_node} {arrow_shape[self.connector_shape -1]} {dest_node}"


class PenTest(NetBoxModel):
    service = models.ForeignKey(
        to=Service, on_delete=models.CASCADE, related_name="nb_itsm_pentest_reports"
    )
    comments = models.TextField(blank=True)
    status = models.IntegerField(
        "State",
        choices=choices.PenTestChoices,
        help_text="Approved or Reproved",
    )
    date = models.DateField("Execution Date")
    ticket = models.CharField("Ticket", max_length=100)
    report_link = models.CharField("Report link", max_length=100)

    tags = TaggableManager(
        through="extras.TaggedItem",
        related_name="nb_itsm_pentest_set",
    )



from django.db.models import Q
from utilities.choices import ChoiceSet

OBJECT_ASSIGNMENT_MODELS = Q(
    Q(app_label='dcim', model='device') |
    Q(app_label='circuits', model='circuit') |
    Q(app_label='virtualization', model='virtualmachine') |
    Q(app_label='nb_itsm', model='application')
)

class PenTestChoices(ChoiceSet):

    APROVED = 1
    REPROVED = 2

    CHOICES = (
        (APROVED, "Approved"),
        (REPROVED, "Reproved"),
    )

class ShapeChoices(ChoiceSet):

    ROUND_EDGES = 1
    STADIUM = 2
    SUBROUTINE = 3
    CYLINDRICAL = 4
    CIRCLE = 5
    ASYMMETRIC = 6
    RHOMBUS = 7
    HEXAGON = 8
    PARALLELOGRAM = 9
    TRAPEZOID = 10

    CHOICES = (
        (ROUND_EDGES,  'Round Edges'),
        (STADIUM,  'Stadium Shaped'),
        (SUBROUTINE,  "Subroutine Shape"),
        (CYLINDRICAL,  "Cylindrical Shape"),
        (CIRCLE,  "Circle Shape"),
        (ASYMMETRIC, "asymmetric shape"),
        (RHOMBUS,"rhombus"),
        (HEXAGON,"Hexagon"),
        (PARALLELOGRAM ,"Parallelogram"),
        (TRAPEZOID,"Trapezoid"),
    )

class ConnectorChoices(ChoiceSet):
    ARROW = 1
    OPEN = 2
    DOTTED_ARROW = 3
    DOTTED_OPEN = 4

    CHOICES = (
        (ARROW,  'Arrow'),
        (OPEN,  'Open'),
        (DOTTED_ARROW,  "Dotted Arrow"),
        (DOTTED_OPEN,  "Dotted Open"),
    )

class ItilServiceStatusChoices(ChoiceSet):
    key = 'ItilServiceCatalog.service_status'

    CHOICES = [
        ('Requirements', 'Requirements', 'yellow'),
        ('Definition', 'Definition', 'yellow'),
        ('Analysis', 'Analysis', 'yellow'),
        ('Approved', 'Approved', 'yellow'),
        ('Chartered', 'Chartered', 'yellow'),
        ('Design', 'Design', 'yellow'),
        ('Development', 'Development', 'yellow'),
        ('Build', 'Build', 'yellow'),
        ('Test', 'Test', 'yellow'),
        ('Release', 'Release', 'green'),
        ('Operational', 'Operational (live)', 'green'),
        ('Retiring', 'Retiring', 'red'),
        ('Retired', 'Retired', 'red'),
    ]

class ItilServiceTypeChoices(ChoiceSet):
    key = 'ServiceCatalog.service_type'

    CHOICES = [
        ('customer', 'Customer-facing Service', 'green'),
        ('supporting', 'Supporting Service', 'orange'),
        ('experimental', 'Experimental Service', 'red'),
    ]
